"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from enum import Enum
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class GetQRCodeAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    IMAGE_PNG = "image/png"


class PaymentLinks(BaseSDK):
    def create(
        self,
        *,
        account_id: str,
        partner_account_id: str,
        merchant_payment_method_id: str,
        amount: Union[components.Amount, components.AmountTypedDict],
        display: Union[
            components.PaymentLinkDisplayOptions,
            components.PaymentLinkDisplayOptionsTypedDict,
        ],
        sales_tax_amount: Optional[
            Union[components.Amount, components.AmountTypedDict]
        ] = None,
        max_uses: Optional[int] = None,
        expires_on: Optional[datetime] = None,
        customer: Optional[
            Union[
                components.PaymentLinkCustomerOptions,
                components.PaymentLinkCustomerOptionsTypedDict,
            ]
        ] = None,
        payment: Optional[
            Union[
                components.PaymentLinkPaymentDetails,
                components.PaymentLinkPaymentDetailsTypedDict,
            ]
        ] = None,
        payout: Optional[
            Union[
                components.PaymentLinkPayoutDetails,
                components.PaymentLinkPayoutDetailsTypedDict,
            ]
        ] = None,
        line_items: Optional[
            Union[
                components.CreatePaymentLinkLineItems,
                components.CreatePaymentLinkLineItemsTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreatePaymentLinkResponse:
        r"""Create a payment link that allows an end user to make a payment on Moov's hosted payment link page.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param partner_account_id: The partner's Moov account ID.
        :param merchant_payment_method_id: The merchant's preferred payment method ID. Must be a wallet payment method.
        :param amount:
        :param display: Customizable display options for a payment link.
        :param sales_tax_amount:
        :param max_uses: An optional limit on the number of times this payment link can be used.

            **For payouts, `maxUses` is always 1.**
        :param expires_on: An optional expiration date for this payment link.
        :param customer:
        :param payment: Options for payment links used to collect payment.
        :param payout:
        :param line_items: An optional collection of line items for a payment link.
            When line items are provided, their total plus sales tax must equal the payment link amount.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreatePaymentLinkRequest(
            account_id=account_id,
            create_payment_link=components.CreatePaymentLink(
                partner_account_id=partner_account_id,
                merchant_payment_method_id=merchant_payment_method_id,
                amount=utils.get_pydantic_model(amount, components.Amount),
                sales_tax_amount=utils.get_pydantic_model(
                    sales_tax_amount, Optional[components.Amount]
                ),
                max_uses=max_uses,
                expires_on=expires_on,
                display=utils.get_pydantic_model(
                    display, components.PaymentLinkDisplayOptions
                ),
                customer=utils.get_pydantic_model(
                    customer, Optional[components.PaymentLinkCustomerOptions]
                ),
                payment=utils.get_pydantic_model(
                    payment, Optional[components.PaymentLinkPaymentDetails]
                ),
                payout=utils.get_pydantic_model(
                    payout, Optional[components.PaymentLinkPayoutDetails]
                ),
                line_items=utils.get_pydantic_model(
                    line_items, Optional[components.CreatePaymentLinkLineItems]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/payment-links",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreatePaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_payment_link,
                False,
                False,
                "json",
                components.CreatePaymentLink,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreatePaymentLinkResponse(
                result=unmarshal_json_response(components.PaymentLink, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.CreatePaymentLinkErrorData, http_res
            )
            raise errors.CreatePaymentLinkError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        account_id: str,
        partner_account_id: str,
        merchant_payment_method_id: str,
        amount: Union[components.Amount, components.AmountTypedDict],
        display: Union[
            components.PaymentLinkDisplayOptions,
            components.PaymentLinkDisplayOptionsTypedDict,
        ],
        sales_tax_amount: Optional[
            Union[components.Amount, components.AmountTypedDict]
        ] = None,
        max_uses: Optional[int] = None,
        expires_on: Optional[datetime] = None,
        customer: Optional[
            Union[
                components.PaymentLinkCustomerOptions,
                components.PaymentLinkCustomerOptionsTypedDict,
            ]
        ] = None,
        payment: Optional[
            Union[
                components.PaymentLinkPaymentDetails,
                components.PaymentLinkPaymentDetailsTypedDict,
            ]
        ] = None,
        payout: Optional[
            Union[
                components.PaymentLinkPayoutDetails,
                components.PaymentLinkPayoutDetailsTypedDict,
            ]
        ] = None,
        line_items: Optional[
            Union[
                components.CreatePaymentLinkLineItems,
                components.CreatePaymentLinkLineItemsTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreatePaymentLinkResponse:
        r"""Create a payment link that allows an end user to make a payment on Moov's hosted payment link page.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param partner_account_id: The partner's Moov account ID.
        :param merchant_payment_method_id: The merchant's preferred payment method ID. Must be a wallet payment method.
        :param amount:
        :param display: Customizable display options for a payment link.
        :param sales_tax_amount:
        :param max_uses: An optional limit on the number of times this payment link can be used.

            **For payouts, `maxUses` is always 1.**
        :param expires_on: An optional expiration date for this payment link.
        :param customer:
        :param payment: Options for payment links used to collect payment.
        :param payout:
        :param line_items: An optional collection of line items for a payment link.
            When line items are provided, their total plus sales tax must equal the payment link amount.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreatePaymentLinkRequest(
            account_id=account_id,
            create_payment_link=components.CreatePaymentLink(
                partner_account_id=partner_account_id,
                merchant_payment_method_id=merchant_payment_method_id,
                amount=utils.get_pydantic_model(amount, components.Amount),
                sales_tax_amount=utils.get_pydantic_model(
                    sales_tax_amount, Optional[components.Amount]
                ),
                max_uses=max_uses,
                expires_on=expires_on,
                display=utils.get_pydantic_model(
                    display, components.PaymentLinkDisplayOptions
                ),
                customer=utils.get_pydantic_model(
                    customer, Optional[components.PaymentLinkCustomerOptions]
                ),
                payment=utils.get_pydantic_model(
                    payment, Optional[components.PaymentLinkPaymentDetails]
                ),
                payout=utils.get_pydantic_model(
                    payout, Optional[components.PaymentLinkPayoutDetails]
                ),
                line_items=utils.get_pydantic_model(
                    line_items, Optional[components.CreatePaymentLinkLineItems]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/payment-links",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreatePaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_payment_link,
                False,
                False,
                "json",
                components.CreatePaymentLink,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createPaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreatePaymentLinkResponse(
                result=unmarshal_json_response(components.PaymentLink, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.CreatePaymentLinkErrorData, http_res
            )
            raise errors.CreatePaymentLinkError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        types: Optional[List[components.PaymentLinkType]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListPaymentLinksResponse:
        r"""List all the payment links created under a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id: The merchant account ID.
        :param skip:
        :param count:
        :param types: A comma-separated list of payment link types to filter results.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListPaymentLinksRequest(
            skip=skip,
            count=count,
            types=types,
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/payment-links",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListPaymentLinksGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listPaymentLinks",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListPaymentLinksResponse(
                result=unmarshal_json_response(List[components.PaymentLink], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        types: Optional[List[components.PaymentLinkType]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListPaymentLinksResponse:
        r"""List all the payment links created under a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id: The merchant account ID.
        :param skip:
        :param count:
        :param types: A comma-separated list of payment link types to filter results.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListPaymentLinksRequest(
            skip=skip,
            count=count,
            types=types,
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/payment-links",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListPaymentLinksGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listPaymentLinks",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListPaymentLinksResponse(
                result=unmarshal_json_response(List[components.PaymentLink], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetPaymentLinkResponse:
        r"""Retrieve a payment link by code.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetPaymentLinkRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetPaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetPaymentLinkResponse(
                result=unmarshal_json_response(components.PaymentLink, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetPaymentLinkResponse:
        r"""Retrieve a payment link by code.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.read` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetPaymentLinkRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetPaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetPaymentLinkResponse(
                result=unmarshal_json_response(components.PaymentLink, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        amount: Optional[
            Union[components.AmountUpdate, components.AmountUpdateTypedDict]
        ] = None,
        sales_tax_amount: Optional[
            Union[components.AmountUpdate, components.AmountUpdateTypedDict]
        ] = None,
        expires_on: OptionalNullable[datetime] = UNSET,
        display: Optional[
            Union[
                components.PaymentLinkDisplayOptionsUpdate,
                components.PaymentLinkDisplayOptionsUpdateTypedDict,
            ]
        ] = None,
        customer: Optional[
            Union[
                components.PaymentLinkCustomerOptions,
                components.PaymentLinkCustomerOptionsTypedDict,
            ]
        ] = None,
        payment: Optional[
            Union[
                components.PaymentLinkPaymentDetailsUpdate,
                components.PaymentLinkPaymentDetailsUpdateTypedDict,
            ]
        ] = None,
        payout: Optional[
            Union[
                components.PaymentLinkPayoutDetailsUpdate,
                components.PaymentLinkPayoutDetailsUpdateTypedDict,
            ]
        ] = None,
        line_items: Optional[
            Union[
                components.CreatePaymentLinkLineItemsUpdate,
                components.CreatePaymentLinkLineItemsUpdateTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdatePaymentLinkResponse:
        r"""Update a payment link.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param amount:
        :param sales_tax_amount:
        :param expires_on:
        :param display: Customizable display options for a payment link.
        :param customer:
        :param payment: Options for payment links used to collect payment.
        :param payout:
        :param line_items: An optional collection of line items for a payment link.
            When line items are provided, their total plus sales tax must equal the payment link amount.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdatePaymentLinkRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
            update_payment_link=components.UpdatePaymentLink(
                amount=utils.get_pydantic_model(
                    amount, Optional[components.AmountUpdate]
                ),
                sales_tax_amount=utils.get_pydantic_model(
                    sales_tax_amount, Optional[components.AmountUpdate]
                ),
                expires_on=expires_on,
                display=utils.get_pydantic_model(
                    display, Optional[components.PaymentLinkDisplayOptionsUpdate]
                ),
                customer=utils.get_pydantic_model(
                    customer, Optional[components.PaymentLinkCustomerOptions]
                ),
                payment=utils.get_pydantic_model(
                    payment, Optional[components.PaymentLinkPaymentDetailsUpdate]
                ),
                payout=utils.get_pydantic_model(
                    payout, Optional[components.PaymentLinkPayoutDetailsUpdate]
                ),
                line_items=utils.get_pydantic_model(
                    line_items, Optional[components.CreatePaymentLinkLineItemsUpdate]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdatePaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_payment_link,
                False,
                False,
                "json",
                components.UpdatePaymentLink,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdatePaymentLinkResponse(
                result=unmarshal_json_response(components.PaymentLink, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpdatePaymentLinkErrorData, http_res
            )
            raise errors.UpdatePaymentLinkError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        amount: Optional[
            Union[components.AmountUpdate, components.AmountUpdateTypedDict]
        ] = None,
        sales_tax_amount: Optional[
            Union[components.AmountUpdate, components.AmountUpdateTypedDict]
        ] = None,
        expires_on: OptionalNullable[datetime] = UNSET,
        display: Optional[
            Union[
                components.PaymentLinkDisplayOptionsUpdate,
                components.PaymentLinkDisplayOptionsUpdateTypedDict,
            ]
        ] = None,
        customer: Optional[
            Union[
                components.PaymentLinkCustomerOptions,
                components.PaymentLinkCustomerOptionsTypedDict,
            ]
        ] = None,
        payment: Optional[
            Union[
                components.PaymentLinkPaymentDetailsUpdate,
                components.PaymentLinkPaymentDetailsUpdateTypedDict,
            ]
        ] = None,
        payout: Optional[
            Union[
                components.PaymentLinkPayoutDetailsUpdate,
                components.PaymentLinkPayoutDetailsUpdateTypedDict,
            ]
        ] = None,
        line_items: Optional[
            Union[
                components.CreatePaymentLinkLineItemsUpdate,
                components.CreatePaymentLinkLineItemsUpdateTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdatePaymentLinkResponse:
        r"""Update a payment link.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param amount:
        :param sales_tax_amount:
        :param expires_on:
        :param display: Customizable display options for a payment link.
        :param customer:
        :param payment: Options for payment links used to collect payment.
        :param payout:
        :param line_items: An optional collection of line items for a payment link.
            When line items are provided, their total plus sales tax must equal the payment link amount.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdatePaymentLinkRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
            update_payment_link=components.UpdatePaymentLink(
                amount=utils.get_pydantic_model(
                    amount, Optional[components.AmountUpdate]
                ),
                sales_tax_amount=utils.get_pydantic_model(
                    sales_tax_amount, Optional[components.AmountUpdate]
                ),
                expires_on=expires_on,
                display=utils.get_pydantic_model(
                    display, Optional[components.PaymentLinkDisplayOptionsUpdate]
                ),
                customer=utils.get_pydantic_model(
                    customer, Optional[components.PaymentLinkCustomerOptions]
                ),
                payment=utils.get_pydantic_model(
                    payment, Optional[components.PaymentLinkPaymentDetailsUpdate]
                ),
                payout=utils.get_pydantic_model(
                    payout, Optional[components.PaymentLinkPayoutDetailsUpdate]
                ),
                line_items=utils.get_pydantic_model(
                    line_items, Optional[components.CreatePaymentLinkLineItemsUpdate]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdatePaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_payment_link,
                False,
                False,
                "json",
                components.UpdatePaymentLink,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updatePaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.UpdatePaymentLinkResponse(
                result=unmarshal_json_response(components.PaymentLink, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.UpdatePaymentLinkErrorData, http_res
            )
            raise errors.UpdatePaymentLinkError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def disable(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.DisablePaymentLinkResponse:
        r"""Disable a payment link.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.DisablePaymentLinkRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
        )

        req = self._build_request(
            method="DELETE",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            _globals=operations.DisablePaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="disablePaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "204", "*"):
            return operations.DisablePaymentLinkResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def disable_async(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.DisablePaymentLinkResponse:
        r"""Disable a payment link.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.DisablePaymentLinkRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="*/*",
            http_headers=http_headers,
            _globals=operations.DisablePaymentLinkGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="disablePaymentLink",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "204", "*"):
            return operations.DisablePaymentLinkResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_qr_code(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        accept_header_override: Optional[GetQRCodeAcceptEnum] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetPaymentLinkQRCodeResponse:
        r"""Retrieve the payment link encoded in a QR code.

        Use the `Accept` header to specify the format of the response. Supported formats are `application/json` and `image/png`.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetPaymentLinkQRCodeRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}/qrcode",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value=accept_header_override.value
            if accept_header_override is not None
            else "application/json;q=1, image/png;q=0",
            http_headers=http_headers,
            _globals=operations.GetPaymentLinkQRCodeGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPaymentLinkQRCode",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return operations.GetPaymentLinkQRCodeResponse(
                result=unmarshal_json_response(
                    components.QRCode, http_res, http_res_text
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "200", "image/png"):
            return operations.GetPaymentLinkQRCodeResponse(
                result=http_res, headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError("Unexpected response received", http_res, http_res_text)

    async def get_qr_code_async(
        self,
        *,
        account_id: str,
        payment_link_code: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        accept_header_override: Optional[GetQRCodeAcceptEnum] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetPaymentLinkQRCodeResponse:
        r"""Retrieve the payment link encoded in a QR code.

        Use the `Accept` header to specify the format of the response. Supported formats are `application/json` and `image/png`.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/transfers.write` scope.

        :param account_id: The merchant account ID.
        :param payment_link_code:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetPaymentLinkQRCodeRequest(
            account_id=account_id,
            payment_link_code=payment_link_code,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/payment-links/{paymentLinkCode}/qrcode",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value=accept_header_override.value
            if accept_header_override is not None
            else "application/json;q=1, image/png;q=0",
            http_headers=http_headers,
            _globals=operations.GetPaymentLinkQRCodeGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getPaymentLinkQRCode",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return operations.GetPaymentLinkQRCodeResponse(
                result=unmarshal_json_response(
                    components.QRCode, http_res, http_res_text
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "200", "image/png"):
            return operations.GetPaymentLinkQRCodeResponse(
                result=http_res, headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError("Unexpected response received", http_res, http_res_text)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BankAccountVerificationStatus(str, Enum):
    NEW = "new"
    SENT_CREDIT = "sent-credit"
    MAX_ATTEMPTS_EXCEEDED = "max-attempts-exceeded"
    FAILED = "failed"
    EXPIRED = "expired"
    SUCCESSFUL = "successful"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CardIssuingNetwork(str, Enum):
    r"""The name of the network a card transaction is routed through."""

    DISCOVER = "discover"
    SHAZAM = "shazam"
    VISA = "visa"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookeventtype import WebhookEventType
from .webhookstatus import WebhookStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookTypedDict(TypedDict):
    r"""A configured webhook endpoint that receives event notifications."""

    webhook_id: str
    r"""Unique identifier for the webhook."""
    url: str
    r"""The URL where webhook events will be sent."""
    status: WebhookStatus
    r"""The status of a webhook."""
    event_types: List[WebhookEventType]
    r"""The list of event types this webhook is subscribed to."""
    description: str
    r"""A description of the webhook for reference."""
    created_on: datetime
    r"""Timestamp of when the webhook was created."""
    updated_on: datetime
    r"""Timestamp of when the webhook was last updated."""
    last_used_on: NotRequired[datetime]
    r"""Timestamp of when the webhook last received an event."""


class Webhook(BaseModel):
    r"""A configured webhook endpoint that receives event notifications."""

    webhook_id: Annotated[str, pydantic.Field(alias="webhookID")]
    r"""Unique identifier for the webhook."""

    url: str
    r"""The URL where webhook events will be sent."""

    status: WebhookStatus
    r"""The status of a webhook."""

    event_types: Annotated[List[WebhookEventType], pydantic.Field(alias="eventTypes")]
    r"""The list of event types this webhook is subscribed to."""

    description: str
    r"""A description of the webhook for reference."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]
    r"""Timestamp of when the webhook was created."""

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]
    r"""Timestamp of when the webhook was last updated."""

    last_used_on: Annotated[Optional[datetime], pydantic.Field(alias="lastUsedOn")] = (
        None
    )
    r"""Timestamp of when the webhook last received an event."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["lastUsedOn"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Webhook.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccountexception import BankAccountException, BankAccountExceptionTypedDict
from .bankaccountstatus import BankAccountStatus
from .bankaccountstatusreason import BankAccountStatusReason
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookDataBankAccountUpdatedTypedDict(TypedDict):
    bank_account_id: str
    account_id: str
    status: BankAccountStatus
    status_reason: BankAccountStatusReason
    r"""The reason the bank account status changed to the current value."""
    exception_details: NotRequired[BankAccountExceptionTypedDict]
    r"""Reason for, and details related to, an `errored` or `verificationFailed` bank account status."""


class WebhookDataBankAccountUpdated(BaseModel):
    bank_account_id: Annotated[str, pydantic.Field(alias="bankAccountID")]

    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    status: BankAccountStatus

    status_reason: Annotated[
        BankAccountStatusReason, pydantic.Field(alias="statusReason")
    ]
    r"""The reason the bank account status changed to the current value."""

    exception_details: Annotated[
        Optional[BankAccountException], pydantic.Field(alias="exceptionDetails")
    ] = None
    r"""Reason for, and details related to, an `errored` or `verificationFailed` bank account status."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["exceptionDetails"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    WebhookDataBankAccountUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .microdepositstatus import MicroDepositStatus
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class CompletedMicroDepositsTypedDict(TypedDict):
    status: MicroDepositStatus


class CompletedMicroDeposits(BaseModel):
    status: MicroDepositStatus



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcountandamount import BillingCountAndAmount, BillingCountAndAmountTypedDict
from .billinginterchangeprogramfee import (
    BillingInterchangeProgramFee,
    BillingInterchangeProgramFeeTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CardBrandFeesTypedDict(TypedDict):
    total: BillingCountAndAmountTypedDict
    r"""Total fees for this card brand."""
    interchange: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of interchange fees."""
    interchange_programs: NotRequired[List[BillingInterchangeProgramFeeTypedDict]]
    r"""A list of interchange program fees."""
    network_passthrough: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of network passthrough fees."""
    completed_domestic_online: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of completed domestic online transactions."""
    completed_domestic_in_person: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of completed domestic in-person transactions."""
    completed_international_online: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of completed international online transactions."""
    completed_international_in_person: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of completed international in-person transactions."""
    declines: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of declined transactions."""
    refunds: NotRequired[BillingCountAndAmountTypedDict]
    r"""Details of refunded transactions."""


class CardBrandFees(BaseModel):
    total: BillingCountAndAmount
    r"""Total fees for this card brand."""

    interchange: Optional[BillingCountAndAmount] = None
    r"""Details of interchange fees."""

    interchange_programs: Annotated[
        Optional[List[BillingInterchangeProgramFee]],
        pydantic.Field(alias="interchangePrograms"),
    ] = None
    r"""A list of interchange program fees."""

    network_passthrough: Annotated[
        Optional[BillingCountAndAmount], pydantic.Field(alias="networkPassthrough")
    ] = None
    r"""Details of network passthrough fees."""

    completed_domestic_online: Annotated[
        Optional[BillingCountAndAmount], pydantic.Field(alias="completedDomesticOnline")
    ] = None
    r"""Details of completed domestic online transactions."""

    completed_domestic_in_person: Annotated[
        Optional[BillingCountAndAmount],
        pydantic.Field(alias="completedDomesticInPerson"),
    ] = None
    r"""Details of completed domestic in-person transactions."""

    completed_international_online: Annotated[
        Optional[BillingCountAndAmount],
        pydantic.Field(alias="completedInternationalOnline"),
    ] = None
    r"""Details of completed international online transactions."""

    completed_international_in_person: Annotated[
        Optional[BillingCountAndAmount],
        pydantic.Field(alias="completedInternationalInPerson"),
    ] = None
    r"""Details of completed international in-person transactions."""

    declines: Optional[BillingCountAndAmount] = None
    r"""Details of declined transactions."""

    refunds: Optional[BillingCountAndAmount] = None
    r"""Details of refunded transactions."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "interchange",
                "interchangePrograms",
                "networkPassthrough",
                "completedDomesticOnline",
                "completedDomesticInPerson",
                "completedInternationalOnline",
                "completedInternationalInPerson",
                "declines",
                "refunds",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CardBrandFees.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .assignproductimage import AssignProductImage, AssignProductImageTypedDict
from .createproductoptiongroup import (
    CreateProductOptionGroup,
    CreateProductOptionGroupTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProductRequestTypedDict(TypedDict):
    r"""Request to create or update a product."""

    title: str
    base_price: AmountDecimalTypedDict
    r"""A product's starting price, before applying modifiers."""
    description: NotRequired[str]
    r"""A detailed description of the product.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """
    images: NotRequired[List[AssignProductImageTypedDict]]
    r"""Assign previously uploaded images to a product or option."""
    option_groups: NotRequired[List[CreateProductOptionGroupTypedDict]]
    r"""Optional configuration options for a product, such as size or color."""


class ProductRequest(BaseModel):
    r"""Request to create or update a product."""

    title: str

    base_price: Annotated[AmountDecimal, pydantic.Field(alias="basePrice")]
    r"""A product's starting price, before applying modifiers."""

    description: Optional[str] = None
    r"""A detailed description of the product.

    - Must be valid UTF-8 text
    - Supports Markdown for formatting
    - HTML is not permitted and will be rejected
    """

    images: Optional[List[AssignProductImage]] = None
    r"""Assign previously uploaded images to a product or option."""

    option_groups: Annotated[
        Optional[List[CreateProductOptionGroup]], pydantic.Field(alias="optionGroups")
    ] = None
    r"""Optional configuration options for a product, such as size or color."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "images", "optionGroups"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ProductRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .applepayresponse import ApplePayResponse, ApplePayResponseTypedDict
from enum import Enum
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ApplePayTransferPaymentMethodPaymentMethodType(str, Enum):
    APPLE_PAY = "apple-pay"


class ApplePayTransferPaymentMethodTypedDict(TypedDict):
    payment_method_id: str
    r"""ID of the payment method."""
    payment_method_type: ApplePayTransferPaymentMethodPaymentMethodType
    apple_pay: ApplePayResponseTypedDict
    r"""Describes an Apple Pay token on a Moov account."""


class ApplePayTransferPaymentMethod(BaseModel):
    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]
    r"""ID of the payment method."""

    payment_method_type: Annotated[
        ApplePayTransferPaymentMethodPaymentMethodType,
        pydantic.Field(alias="paymentMethodType"),
    ]

    apple_pay: Annotated[ApplePayResponse, pydantic.Field(alias="applePay")]
    r"""Describes an Apple Pay token on a Moov account."""


try:
    ApplePayTransferPaymentMethod.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class WebhookSecretTypedDict(TypedDict):
    r"""The secret used to verify webhook payloads."""

    secret: str
    r"""The secret key used to sign webhook payloads. Use this to verify the authenticity of incoming webhooks."""


class WebhookSecret(BaseModel):
    r"""The secret used to verify webhook payloads."""

    secret: str
    r"""The secret key used to sign webhook payloads. Use this to verify the authenticity of incoming webhooks."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DocumentType(str, Enum):
    r"""Types of documents that can be uploaded."""

    DRIVERS_LICENSE = "driversLicense"
    PASSPORT = "passport"
    UTILITY_BILL = "utilityBill"
    BANK_STATEMENT = "bankStatement"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class EvidenceType(str, Enum):
    RECEIPT = "receipt"
    PROOF_OF_DELIVERY = "proof-of-delivery"
    CANCELATION_POLICY = "cancelation-policy"
    TERMS_OF_SERVICE = "terms-of-service"
    CUSTOMER_COMMUNICATION = "customer-communication"
    GENERIC_EVIDENCE = "generic-evidence"
    COVER_LETTER = "cover-letter"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .disbursementpaymentmethodtype import DisbursementPaymentMethodType
from .payoutrecipient import PayoutRecipient, PayoutRecipientTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentLinkPayoutDetailsUpdateTypedDict(TypedDict):
    allowed_methods: NotRequired[List[DisbursementPaymentMethodType]]
    r"""A list of payment methods that should be supported for this payment link."""
    recipient: NotRequired[PayoutRecipientTypedDict]
    r"""Specify the intended recipient of the payout.
    Either `email` or `phone` must be specified, but not both.

    This information will be used to authenticate the end user when they follow the payment link.
    """
    metadata: NotRequired[Dict[str, str]]
    r"""Optional free-form metadata for the transfer."""


class PaymentLinkPayoutDetailsUpdate(BaseModel):
    allowed_methods: Annotated[
        Optional[List[DisbursementPaymentMethodType]],
        pydantic.Field(alias="allowedMethods"),
    ] = None
    r"""A list of payment methods that should be supported for this payment link."""

    recipient: Optional[PayoutRecipient] = None
    r"""Specify the intended recipient of the payout.
    Either `email` or `phone` must be specified, but not both.

    This information will be used to authenticate the end user when they follow the payment link.
    """

    metadata: Optional[Dict[str, str]] = None
    r"""Optional free-form metadata for the transfer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["allowedMethods", "recipient", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PaymentLinkPayoutDetailsUpdate.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createscheduledtransferlineitem import (
    CreateScheduledTransferLineItem,
    CreateScheduledTransferLineItemTypedDict,
)
from moovio_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class CreateScheduledTransferLineItemsTypedDict(TypedDict):
    r"""An optional collection of line items for a scheduled transfer.
    When line items are provided their total must equal `amount` minus `salesTaxAmount`.
    """

    items: List[CreateScheduledTransferLineItemTypedDict]
    r"""The list of line items."""


class CreateScheduledTransferLineItems(BaseModel):
    r"""An optional collection of line items for a scheduled transfer.
    When line items are provided their total must equal `amount` minus `salesTaxAmount`.
    """

    items: List[CreateScheduledTransferLineItem]
    r"""The list of line items."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentLinkCustomerOptionsTypedDict(TypedDict):
    require_address: NotRequired[bool]
    r"""If true, a billing address is required when completing the payment form."""
    require_phone: NotRequired[bool]
    r"""If true, a phone number is required when completing the payment form."""
    metadata: NotRequired[Dict[str, str]]
    r"""Optional free-form metadata for the Moov account that will represent this customer."""


class PaymentLinkCustomerOptions(BaseModel):
    require_address: Annotated[
        Optional[bool], pydantic.Field(alias="requireAddress")
    ] = None
    r"""If true, a billing address is required when completing the payment form."""

    require_phone: Annotated[Optional[bool], pydantic.Field(alias="requirePhone")] = (
        None
    )
    r"""If true, a phone number is required when completing the payment form."""

    metadata: Optional[Dict[str, str]] = None
    r"""Optional free-form metadata for the Moov account that will represent this customer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["requireAddress", "requirePhone", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PaymentLinkCustomerOptions.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CallToAction(str, Enum):
    r"""The text to be displayed on web form's submit button.

    If set to \"auto\" the UI will automatically select between
    \"pay\" and \"confirm\" for payments and payouts respectively.
    """

    PAY = "pay"
    BOOK = "book"
    SUBSCRIBE = "subscribe"
    DONATE = "donate"
    CONFIRM = "confirm"
    AUTO = "auto"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookdatatransferstatus import WebhookDataTransferStatus
from .webhooktransferpaymentmethoddetails import (
    WebhookTransferPaymentMethodDetails,
    WebhookTransferPaymentMethodDetailsTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WebhookDataTransferUpdatedTypedDict(TypedDict):
    account_id: str
    r"""The accountID which facilitated the transfer."""
    transfer_id: str
    status: WebhookDataTransferStatus
    source: WebhookTransferPaymentMethodDetailsTypedDict
    r"""Payment method details for the source or destination of a transfer."""
    destination: WebhookTransferPaymentMethodDetailsTypedDict
    r"""Payment method details for the source or destination of a transfer."""
    foreign_id: NotRequired[str]


class WebhookDataTransferUpdated(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]
    r"""The accountID which facilitated the transfer."""

    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]

    status: WebhookDataTransferStatus

    source: WebhookTransferPaymentMethodDetails
    r"""Payment method details for the source or destination of a transfer."""

    destination: WebhookTransferPaymentMethodDetails
    r"""Payment method details for the source or destination of a transfer."""

    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["foreignID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    WebhookDataTransferUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class ListTransfersValidationErrorData(BaseModel):
    account_i_ds: Annotated[Optional[str], pydantic.Field(alias="accountIDs")] = None
    status: Optional[str] = None
    start_date_time: Annotated[Optional[str], pydantic.Field(alias="startDateTime")] = (
        None
    )
    end_date_time: Annotated[Optional[str], pydantic.Field(alias="endDateTime")] = None
    skip: Optional[str] = None
    count: Optional[str] = None
    group_id: Annotated[Optional[str], pydantic.Field(alias="groupID")] = None
    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None
    schedule_id: Annotated[Optional[str], pydantic.Field(alias="scheduleID")] = None
    payment_link_code: Annotated[
        Optional[str], pydantic.Field(alias="paymentLinkCode")
    ] = None
    refunded: Optional[str] = None
    disputed: Optional[str] = None


@dataclass(unsafe_hash=True)
class ListTransfersValidationError(MoovError):
    data: ListTransfersValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: ListTransfersValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RTPRejectionCode(str, Enum):
    r"""The rejection code of an RTP transaction that caused the bank account status to change.

    - AC03: Account Invalid
    - AC04: Account Closed
    - AC06: Account Blocked
    - AC14: Creditor Account Type Invalid
    - AG01: Transactions Forbidden On Account
    - AG03: Transaction Type Not Supported
    - MD07: Customer Deceased
    """

    AC03 = "AC03"
    AC04 = "AC04"
    AC06 = "AC06"
    AC14 = "AC14"
    AG01 = "AG01"
    AG03 = "AG03"
    MD07 = "MD07"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webhookeventtype import WebhookEventType
from .webhookstatus import WebhookStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class UpdateWebhookTypedDict(TypedDict):
    r"""Request body for updating an existing webhook."""

    url: str
    r"""The URL where webhook events will be sent."""
    status: WebhookStatus
    r"""The status of the webhook."""
    event_types: List[WebhookEventType]
    r"""The list of event types this webhook should subscribe to."""
    description: str
    r"""A description of the webhook for reference. Can be an empty string."""


class UpdateWebhook(BaseModel):
    r"""Request body for updating an existing webhook."""

    url: str
    r"""The URL where webhook events will be sent."""

    status: WebhookStatus
    r"""The status of the webhook."""

    event_types: Annotated[List[WebhookEventType], pydantic.Field(alias="eventTypes")]
    r"""The list of event types this webhook should subscribe to."""

    description: str
    r"""A description of the webhook for reference. Can be an empty string."""


try:
    UpdateWebhook.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FacilitatorFeeTypedDict(TypedDict):
    r"""Total or markup fee."""

    total: NotRequired[int]
    r"""Total facilitator fee in cents. Only either `total` or `totalDecimal` can be set."""
    total_decimal: NotRequired[str]
    r"""Same as `total`, but a decimal-formatted numerical string that represents up to 9 decimal place precision.

    Only either `total` or `totalDecimal` can be set. Set this field if you expect the fee to be in fractions of a cent.
    """
    markup: NotRequired[int]
    r"""Markup facilitator fee in cents. Only either `markup` or `markupDecimal` can be set."""
    markup_decimal: NotRequired[str]
    r"""Same as `markup`, but a decimal-formatted numerical string that represents up to 9 decimal place precision.
    Only either `markup` or `markupDecimal` can be set. Set this field if you expect the fee to be in fractions of a cent.
    """


class FacilitatorFee(BaseModel):
    r"""Total or markup fee."""

    total: Optional[int] = None
    r"""Total facilitator fee in cents. Only either `total` or `totalDecimal` can be set."""

    total_decimal: Annotated[Optional[str], pydantic.Field(alias="totalDecimal")] = None
    r"""Same as `total`, but a decimal-formatted numerical string that represents up to 9 decimal place precision.

    Only either `total` or `totalDecimal` can be set. Set this field if you expect the fee to be in fractions of a cent.
    """

    markup: Optional[int] = None
    r"""Markup facilitator fee in cents. Only either `markup` or `markupDecimal` can be set."""

    markup_decimal: Annotated[Optional[str], pydantic.Field(alias="markupDecimal")] = (
        None
    )
    r"""Same as `markup`, but a decimal-formatted numerical string that represents up to 9 decimal place precision.
    Only either `markup` or `markupDecimal` can be set. Set this field if you expect the fee to be in fractions of a cent.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["total", "totalDecimal", "markup", "markupDecimal"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    FacilitatorFee.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentmethodscard import PaymentMethodsCard, PaymentMethodsCardTypedDict
from enum import Enum
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class PullFromCardPaymentMethodPaymentMethodType(str, Enum):
    PULL_FROM_CARD = "pull-from-card"


class PullFromCardPaymentMethodTypedDict(TypedDict):
    payment_method_id: str
    r"""ID of the payment method."""
    payment_method_type: PullFromCardPaymentMethodPaymentMethodType
    card: PaymentMethodsCardTypedDict
    r"""A card as contained within a payment method."""


class PullFromCardPaymentMethod(BaseModel):
    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]
    r"""ID of the payment method."""

    payment_method_type: Annotated[
        PullFromCardPaymentMethodPaymentMethodType,
        pydantic.Field(alias="paymentMethodType"),
    ]

    card: PaymentMethodsCard
    r"""A card as contained within a payment method."""


try:
    PullFromCardPaymentMethod.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    createaccountsettings as components_createaccountsettings,
    createprofileerror as components_createprofileerror,
    customersupporterror as components_customersupporterror,
    termsofserviceerror as components_termsofserviceerror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class PatchAccountErrorData(BaseModel):
    profile: Optional[components_createprofileerror.CreateProfileError] = None
    metadata: Optional[str] = None
    terms_of_service: Annotated[
        Optional[components_termsofserviceerror.TermsOfServiceError],
        pydantic.Field(alias="termsOfService"),
    ] = None
    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None
    customer_support: Annotated[
        Optional[components_customersupporterror.CustomerSupportError],
        pydantic.Field(alias="customerSupport"),
    ] = None
    settings: Optional[components_createaccountsettings.CreateAccountSettings] = None
    error: Optional[str] = None


@dataclass(unsafe_hash=True)
class PatchAccountError(MoovError):
    data: PatchAccountErrorData = field(hash=False)

    def __init__(
        self,
        data: PatchAccountErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreatePaymentLinkLineItemOptionTypedDict(TypedDict):
    r"""Represents a modifier or option applied to a line item."""

    name: str
    r"""The name of the option or modifier."""
    quantity: int
    r"""The quantity of this option."""
    price_modifier: NotRequired[AmountDecimalTypedDict]
    r"""Optional price modification applied by this option. Can be positive, negative, or zero."""
    image_i_ds: NotRequired[List[str]]
    r"""Optional list of images associated with this line item option.
    This field is being deprecated in favor using the images associated with a productID and will soon be unsupported.
    """
    group: NotRequired[str]
    r"""Optional group identifier to categorize related options (e.g., 'toppings')."""


class CreatePaymentLinkLineItemOption(BaseModel):
    r"""Represents a modifier or option applied to a line item."""

    name: str
    r"""The name of the option or modifier."""

    quantity: int
    r"""The quantity of this option."""

    price_modifier: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="priceModifier")
    ] = None
    r"""Optional price modification applied by this option. Can be positive, negative, or zero."""

    image_i_ds: Annotated[
        Optional[List[str]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="imageIDs",
        ),
    ] = None
    r"""Optional list of images associated with this line item option.
    This field is being deprecated in favor using the images associated with a productID and will soon be unsupported.
    """

    group: Optional[str] = None
    r"""Optional group identifier to categorize related options (e.g., 'toppings')."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["priceModifier", "imageIDs", "group"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreatePaymentLinkLineItemOption.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .sendfundsach import SendFundsAch, SendFundsAchTypedDict
from .sendfundsinstantbank import SendFundsInstantBank, SendFundsInstantBankTypedDict
from .sendfundspushtocard import SendFundsPushToCard, SendFundsPushToCardTypedDict
from .sendfundsrtp import SendFundsRtp, SendFundsRtpTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SendFundsTypedDict(TypedDict):
    ach: NotRequired[SendFundsAchTypedDict]
    push_to_card: NotRequired[SendFundsPushToCardTypedDict]
    rtp: NotRequired[SendFundsRtpTypedDict]
    instant_bank: NotRequired[SendFundsInstantBankTypedDict]


class SendFunds(BaseModel):
    ach: Optional[SendFundsAch] = None

    push_to_card: Annotated[
        Optional[SendFundsPushToCard], pydantic.Field(alias="pushToCard")
    ] = None

    rtp: Optional[SendFundsRtp] = None

    instant_bank: Annotated[
        Optional[SendFundsInstantBank], pydantic.Field(alias="instantBank")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["ach", "pushToCard", "rtp", "instantBank"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    SendFunds.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .manualtermsofservice import ManualTermsOfService, ManualTermsOfServiceTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class TermsOfServicePayloadTypedDict(TypedDict):
    token: NotRequired[str]
    manual: NotRequired[ManualTermsOfServiceTypedDict]
    r"""Describes the acceptance of the Terms of Service. All data is required, and must be from the user."""


class TermsOfServicePayload(BaseModel):
    token: Optional[str] = None

    manual: Optional[ManualTermsOfService] = None
    r"""Describes the acceptance of the Terms of Service. All data is required, and must be from the user."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["token", "manual"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ACHTransactionStatus(str, Enum):
    r"""Status of a transaction within the ACH lifecycle."""

    UNKNOWN = ""
    INITIATED = "initiated"
    ORIGINATED = "originated"
    CORRECTED = "corrected"
    RETURNED = "returned"
    COMPLETED = "completed"
    CANCELED = "canceled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accountcapability import AccountCapability, AccountCapabilityTypedDict
from .accounttype import AccountType
from .customersupport import CustomerSupport, CustomerSupportTypedDict
from .mode import Mode
from .profile import Profile, ProfileTypedDict
from .settings import Settings, SettingsTypedDict
from .termsofservice import TermsOfService, TermsOfServiceTypedDict
from .verification import Verification, VerificationTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AccountTypedDict(TypedDict):
    account_id: str
    r"""Unique identifier for this account."""
    mode: Mode
    r"""The operating mode for an account."""
    account_type: AccountType
    r"""The type of entity represented by this account."""
    display_name: str
    profile: ProfileTypedDict
    r"""Describes a Moov account profile. A profile will have a business, individual, or guest depending on the account's type."""
    verification: VerificationTypedDict
    r"""Describes identity verification status and relevant identity verification documents."""
    created_on: datetime
    updated_on: datetime
    metadata: NotRequired[Dict[str, str]]
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""
    terms_of_service: NotRequired[TermsOfServiceTypedDict]
    r"""Describes the acceptance of the Terms of Service."""
    capabilities: NotRequired[List[AccountCapabilityTypedDict]]
    foreign_id: NotRequired[str]
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""
    customer_support: NotRequired[CustomerSupportTypedDict]
    r"""User-provided information that can be displayed on credit card transactions for customers to use when
    contacting a customer support team. This data is only allowed on a business account.
    """
    settings: NotRequired[SettingsTypedDict]
    r"""User provided settings to manage an account."""
    disconnected_on: NotRequired[datetime]


class Account(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]
    r"""Unique identifier for this account."""

    mode: Mode
    r"""The operating mode for an account."""

    account_type: Annotated[AccountType, pydantic.Field(alias="accountType")]
    r"""The type of entity represented by this account."""

    display_name: Annotated[str, pydantic.Field(alias="displayName")]

    profile: Profile
    r"""Describes a Moov account profile. A profile will have a business, individual, or guest depending on the account's type."""

    verification: Verification
    r"""Describes identity verification status and relevant identity verification documents."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]

    metadata: Optional[Dict[str, str]] = None
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""

    terms_of_service: Annotated[
        Optional[TermsOfService], pydantic.Field(alias="termsOfService")
    ] = None
    r"""Describes the acceptance of the Terms of Service."""

    capabilities: Optional[List[AccountCapability]] = None

    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""

    customer_support: Annotated[
        Optional[CustomerSupport], pydantic.Field(alias="customerSupport")
    ] = None
    r"""User-provided information that can be displayed on credit card transactions for customers to use when
    contacting a customer support team. This data is only allowed on a business account.
    """

    settings: Optional[Settings] = None
    r"""User provided settings to manage an account."""

    disconnected_on: Annotated[
        Optional[datetime], pydantic.Field(alias="disconnectedOn")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "metadata",
                "termsOfService",
                "capabilities",
                "foreignID",
                "customerSupport",
                "settings",
                "disconnectedOn",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Account.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .billingsummarydetails import BillingSummaryDetails, BillingSummaryDetailsTypedDict
from .billingsummaryinterchange import (
    BillingSummaryInterchange,
    BillingSummaryInterchangeTypedDict,
)
from .partnerfees import PartnerFees, PartnerFeesTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict, deprecated


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class BillingSummaryVolumeAmountTypedDict(TypedDict):
    r"""The total transaction volume amount. This field is deprecated and will be removed in a future release."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    value_decimal: str
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class BillingSummaryVolumeAmount(BaseModel):
    r"""The total transaction volume amount. This field is deprecated and will be removed in a future release."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    value_decimal: Annotated[str, pydantic.Field(alias="valueDecimal")]
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


class CardAcquiringTypedDict(TypedDict):
    r"""A summary of card acquiring fees."""

    volume_amount: NotRequired[BillingSummaryVolumeAmountTypedDict]
    r"""The total transaction volume amount. This field is deprecated and will be removed in a future release."""
    volume_count: NotRequired[int]
    r"""The total number of transactions. This field is deprecated and will be removed in a future release."""
    fee_amount: NotRequired[AmountDecimalTypedDict]
    r"""The total fee amount."""
    merchant_fees_collected: NotRequired[AmountDecimalTypedDict]
    r"""Total fee revenue collected from merchants."""
    partner_fees_assessed: NotRequired[AmountDecimalTypedDict]
    r"""Total fee costs incurred by the partner."""
    net_income: NotRequired[AmountDecimalTypedDict]
    r"""Net revenue after deducting partner fee costs."""
    interchange_fees: NotRequired[BillingSummaryInterchangeTypedDict]
    r"""A summary of interchange fees by card brand."""


class CardAcquiring(BaseModel):
    r"""A summary of card acquiring fees."""

    volume_amount: Annotated[
        Optional[BillingSummaryVolumeAmount],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="volumeAmount",
        ),
    ] = None
    r"""The total transaction volume amount. This field is deprecated and will be removed in a future release."""

    volume_count: Annotated[
        Optional[int],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="volumeCount",
        ),
    ] = None
    r"""The total number of transactions. This field is deprecated and will be removed in a future release."""

    fee_amount: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="feeAmount")
    ] = None
    r"""The total fee amount."""

    merchant_fees_collected: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="merchantFeesCollected")
    ] = None
    r"""Total fee revenue collected from merchants."""

    partner_fees_assessed: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="partnerFeesAssessed")
    ] = None
    r"""Total fee costs incurred by the partner."""

    net_income: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="netIncome")
    ] = None
    r"""Net revenue after deducting partner fee costs."""

    interchange_fees: Annotated[
        Optional[BillingSummaryInterchange], pydantic.Field(alias="interchangeFees")
    ] = None
    r"""A summary of interchange fees by card brand."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "volumeAmount",
                "volumeCount",
                "feeAmount",
                "merchantFeesCollected",
                "partnerFeesAssessed",
                "netIncome",
                "interchangeFees",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class BillingSummaryPlatformFeesTypedDict(TypedDict):
    r"""The total amount of platform fees. This field is deprecated and will be removed in a future release. Use summary.accountFees."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    value_decimal: str
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class BillingSummaryPlatformFees(BaseModel):
    r"""The total amount of platform fees. This field is deprecated and will be removed in a future release. Use summary.accountFees."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    value_decimal: Annotated[str, pydantic.Field(alias="valueDecimal")]
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class AdjustmentFeesTypedDict(TypedDict):
    r"""The total amount of adjustment fees. This field is deprecated and will be removed in a future release."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    value_decimal: str
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class AdjustmentFees(BaseModel):
    r"""The total amount of adjustment fees. This field is deprecated and will be removed in a future release."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    value_decimal: Annotated[str, pydantic.Field(alias="valueDecimal")]
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class OtherFeesTypedDict(TypedDict):
    r"""The total amount of other card fees. This field is deprecated and will be removed in a future release. Use summary.otherCardFees."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    value_decimal: str
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class OtherFees(BaseModel):
    r"""The total amount of other card fees. This field is deprecated and will be removed in a future release. Use summary.otherCardFees."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    value_decimal: Annotated[str, pydantic.Field(alias="valueDecimal")]
    r"""A decimal-formatted numerical string that represents up to 9 decimal place precision.

    For example, $12.987654321 is '12.987654321'.
    """


class BillingSummaryTypedDict(TypedDict):
    r"""A summary of all fees included in a statement."""

    card_acquiring: NotRequired[CardAcquiringTypedDict]
    r"""A summary of card acquiring fees."""
    ach: NotRequired[BillingSummaryDetailsTypedDict]
    r"""A summary of ACH fees."""
    instant_payments: NotRequired[BillingSummaryDetailsTypedDict]
    r"""A summary of instant payment fees."""
    platform_fees: NotRequired[BillingSummaryPlatformFeesTypedDict]
    r"""The total amount of platform fees. This field is deprecated and will be removed in a future release. Use summary.accountFees."""
    account_fees: NotRequired[BillingSummaryDetailsTypedDict]
    r"""A summary of account fees."""
    adjustment_fees: NotRequired[AdjustmentFeesTypedDict]
    r"""The total amount of adjustment fees. This field is deprecated and will be removed in a future release."""
    other_fees: NotRequired[OtherFeesTypedDict]
    r"""The total amount of other card fees. This field is deprecated and will be removed in a future release. Use summary.otherCardFees."""
    other_card_fees: NotRequired[BillingSummaryDetailsTypedDict]
    r"""A summary of other card fees."""
    total: NotRequired[AmountDecimalTypedDict]
    r"""The total amount of all fees."""
    net_income_subtotal: NotRequired[BillingSummaryDetailsTypedDict]
    r"""Total net revenue after deducting total partner fees."""
    revenue_share: NotRequired[str]
    r"""Partner’s share of the net income, expressed as a percentage."""
    residual_subtotal: NotRequired[AmountDecimalTypedDict]
    r"""The portion of net income allocated to the partner before monthly partner costs."""
    monthly_partner_costs: NotRequired[PartnerFeesTypedDict]
    r"""Monthly partner costs that are charged separately and not included in residual subtotal (e.g. platform fees, minimums)."""
    net_partner_payment: NotRequired[AmountDecimalTypedDict]
    r"""Final partner payment after deducting monthlyPartnerCosts."""


class BillingSummary(BaseModel):
    r"""A summary of all fees included in a statement."""

    card_acquiring: Annotated[
        Optional[CardAcquiring], pydantic.Field(alias="cardAcquiring")
    ] = None
    r"""A summary of card acquiring fees."""

    ach: Optional[BillingSummaryDetails] = None
    r"""A summary of ACH fees."""

    instant_payments: Annotated[
        Optional[BillingSummaryDetails], pydantic.Field(alias="instantPayments")
    ] = None
    r"""A summary of instant payment fees."""

    platform_fees: Annotated[
        Optional[BillingSummaryPlatformFees],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="platformFees",
        ),
    ] = None
    r"""The total amount of platform fees. This field is deprecated and will be removed in a future release. Use summary.accountFees."""

    account_fees: Annotated[
        Optional[BillingSummaryDetails], pydantic.Field(alias="accountFees")
    ] = None
    r"""A summary of account fees."""

    adjustment_fees: Annotated[
        Optional[AdjustmentFees],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="adjustmentFees",
        ),
    ] = None
    r"""The total amount of adjustment fees. This field is deprecated and will be removed in a future release."""

    other_fees: Annotated[
        Optional[OtherFees],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="otherFees",
        ),
    ] = None
    r"""The total amount of other card fees. This field is deprecated and will be removed in a future release. Use summary.otherCardFees."""

    other_card_fees: Annotated[
        Optional[BillingSummaryDetails], pydantic.Field(alias="otherCardFees")
    ] = None
    r"""A summary of other card fees."""

    total: Optional[AmountDecimal] = None
    r"""The total amount of all fees."""

    net_income_subtotal: Annotated[
        Optional[BillingSummaryDetails], pydantic.Field(alias="netIncomeSubtotal")
    ] = None
    r"""Total net revenue after deducting total partner fees."""

    revenue_share: Annotated[Optional[str], pydantic.Field(alias="revenueShare")] = None
    r"""Partner’s share of the net income, expressed as a percentage."""

    residual_subtotal: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="residualSubtotal")
    ] = None
    r"""The portion of net income allocated to the partner before monthly partner costs."""

    monthly_partner_costs: Annotated[
        Optional[PartnerFees], pydantic.Field(alias="monthlyPartnerCosts")
    ] = None
    r"""Monthly partner costs that are charged separately and not included in residual subtotal (e.g. platform fees, minimums)."""

    net_partner_payment: Annotated[
        Optional[AmountDecimal], pydantic.Field(alias="netPartnerPayment")
    ] = None
    r"""Final partner payment after deducting monthlyPartnerCosts."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "cardAcquiring",
                "ach",
                "instantPayments",
                "platformFees",
                "accountFees",
                "adjustmentFees",
                "otherFees",
                "otherCardFees",
                "total",
                "netIncomeSubtotal",
                "revenueShare",
                "residualSubtotal",
                "monthlyPartnerCosts",
                "netPartnerPayment",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    BillingSummaryVolumeAmount.model_rebuild()
except NameError:
    pass
try:
    CardAcquiring.model_rebuild()
except NameError:
    pass
try:
    BillingSummaryPlatformFees.model_rebuild()
except NameError:
    pass
try:
    AdjustmentFees.model_rebuild()
except NameError:
    pass
try:
    OtherFees.model_rebuild()
except NameError:
    pass
try:
    BillingSummary.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PaymentLinkType(str, Enum):
    PAYMENT = "payment"
    PAYOUT = "payout"
    INVOICE_PAYMENT = "invoice-payment"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class TerminalApplicationStatus(str, Enum):
    r"""Status of the terminal application."""

    UNDEFINED = "undefined"
    PENDING = "pending"
    ENABLED = "enabled"
    DISABLED = "disabled"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ReturnPolicyType(str, Enum):
    NONE = "none"
    EXCHANGE_ONLY = "exchangeOnly"
    WITHIN_THIRTY_DAYS = "withinThirtyDays"
    OTHER = "other"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimalupdate import AmountDecimalUpdate, AmountDecimalUpdateTypedDict
from .createinvoicelineitemsupdate import (
    CreateInvoiceLineItemsUpdate,
    CreateInvoiceLineItemsUpdateTypedDict,
)
from .invoicestatus import InvoiceStatus
from datetime import datetime
from moovio_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateInvoiceTypedDict(TypedDict):
    description: NotRequired[str]
    line_items: NotRequired[CreateInvoiceLineItemsUpdateTypedDict]
    r"""A collection of line items for an invoice."""
    invoice_date: NotRequired[Nullable[datetime]]
    due_date: NotRequired[Nullable[datetime]]
    status: NotRequired[InvoiceStatus]
    r"""The status can be updated to one of the following values under specific conditions:
    - `canceled`: Can only be set if the current status is `draft`, `unpaid`, or `overdue`. Canceling an invoice
    indicates the invoice is no longer expected to be paid (e.g., the charge was waived or terms changed).
    Canceled invoices still appear in list results by default and remain part of the invoice history.
    To completely discard an invoice created by mistake, use the delete endpoint instead.
    - `unpaid`: Can only be set if the current status is `draft`. Setting the status to `unpaid` finalizes the invoice and sends an email with a payment link to the customer.
    """
    tax_amount: NotRequired[AmountDecimalUpdateTypedDict]


class UpdateInvoice(BaseModel):
    description: Optional[str] = None

    line_items: Annotated[
        Optional[CreateInvoiceLineItemsUpdate], pydantic.Field(alias="lineItems")
    ] = None
    r"""A collection of line items for an invoice."""

    invoice_date: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="invoiceDate")
    ] = UNSET

    due_date: Annotated[OptionalNullable[datetime], pydantic.Field(alias="dueDate")] = (
        UNSET
    )

    status: Optional[InvoiceStatus] = None
    r"""The status can be updated to one of the following values under specific conditions:
    - `canceled`: Can only be set if the current status is `draft`, `unpaid`, or `overdue`. Canceling an invoice
    indicates the invoice is no longer expected to be paid (e.g., the charge was waived or terms changed).
    Canceled invoices still appear in list results by default and remain part of the invoice history.
    To completely discard an invoice created by mistake, use the delete endpoint instead.
    - `unpaid`: Can only be set if the current status is `draft`. Setting the status to `unpaid` finalizes the invoice and sends an email with a payment link to the customer.
    """

    tax_amount: Annotated[
        Optional[AmountDecimalUpdate], pydantic.Field(alias="taxAmount")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "description",
                "lineItems",
                "invoiceDate",
                "dueDate",
                "status",
                "taxAmount",
            ]
        )
        nullable_fields = set(["invoiceDate", "dueDate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    UpdateInvoice.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .instantbankfailurecode import InstantBankFailureCode
from .instantbanknetwork import InstantBankNetwork
from .instantbanktransactionstatus import InstantBankTransactionStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class InstantBankTransactionDetailsTypedDict(TypedDict):
    r"""Instant-bank specific details about the transaction."""

    network: InstantBankNetwork
    r"""The network that the transaction was processed on."""
    status: InstantBankTransactionStatus
    r"""Status of a transaction within the instant-bank lifecycle."""
    network_response_code: NotRequired[str]
    r"""Response code returned by network on failure."""
    failure_code: NotRequired[InstantBankFailureCode]
    r"""Status codes for instant-bank failures."""
    end_to_end_id: NotRequired[str]
    initiated_on: NotRequired[datetime]
    completed_on: NotRequired[datetime]
    failed_on: NotRequired[datetime]
    accepted_without_posting_on: NotRequired[datetime]


class InstantBankTransactionDetails(BaseModel):
    r"""Instant-bank specific details about the transaction."""

    network: InstantBankNetwork
    r"""The network that the transaction was processed on."""

    status: InstantBankTransactionStatus
    r"""Status of a transaction within the instant-bank lifecycle."""

    network_response_code: Annotated[
        Optional[str], pydantic.Field(alias="networkResponseCode")
    ] = None
    r"""Response code returned by network on failure."""

    failure_code: Annotated[
        Optional[InstantBankFailureCode], pydantic.Field(alias="failureCode")
    ] = None
    r"""Status codes for instant-bank failures."""

    end_to_end_id: Annotated[Optional[str], pydantic.Field(alias="endToEndID")] = None

    initiated_on: Annotated[Optional[datetime], pydantic.Field(alias="initiatedOn")] = (
        None
    )

    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )

    failed_on: Annotated[Optional[datetime], pydantic.Field(alias="failedOn")] = None

    accepted_without_posting_on: Annotated[
        Optional[datetime], pydantic.Field(alias="acceptedWithoutPostingOn")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "networkResponseCode",
                "failureCode",
                "endToEndID",
                "initiatedOn",
                "completedOn",
                "failedOn",
                "acceptedWithoutPostingOn",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    InstantBankTransactionDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ManualTermsOfServiceTypedDict(TypedDict):
    r"""Describes the acceptance of the Terms of Service. All data is required, and must be from the user."""

    accepted_date: datetime
    r"""The date and time the terms of service were accepted."""
    accepted_ip: str
    r"""The IP address from which the terms of service were accepted."""
    accepted_user_agent: str
    r"""The user-agent of the user making the request."""
    accepted_domain: str


class ManualTermsOfService(BaseModel):
    r"""Describes the acceptance of the Terms of Service. All data is required, and must be from the user."""

    accepted_date: Annotated[datetime, pydantic.Field(alias="acceptedDate")]
    r"""The date and time the terms of service were accepted."""

    accepted_ip: Annotated[str, pydantic.Field(alias="acceptedIP")]
    r"""The IP address from which the terms of service were accepted."""

    accepted_user_agent: Annotated[str, pydantic.Field(alias="acceptedUserAgent")]
    r"""The user-agent of the user making the request."""

    accepted_domain: Annotated[str, pydantic.Field(alias="acceptedDomain")]


try:
    ManualTermsOfService.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .mode import Mode
from .occurrencesresponse import OccurrencesResponse, OccurrencesResponseTypedDict
from .partialscheduleaccount import (
    PartialScheduleAccount,
    PartialScheduleAccountTypedDict,
)
from .recurresponse import RecurResponse, RecurResponseTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ScheduleListResponseTypedDict(TypedDict):
    destination_account_id: str
    mode: Mode
    r"""The operating mode for an account."""
    owner_account_id: str
    partner_account_id: str
    schedule_id: str
    source_account_id: str
    created_on: datetime
    updated_on: datetime
    description: NotRequired[str]
    r"""Simple description to place on the transfer."""
    occurrences: NotRequired[List[OccurrencesResponseTypedDict]]
    recur: NotRequired[RecurResponseTypedDict]
    disabled_on: NotRequired[datetime]
    source_account: NotRequired[PartialScheduleAccountTypedDict]
    destination_account: NotRequired[PartialScheduleAccountTypedDict]
    partner_account: NotRequired[PartialScheduleAccountTypedDict]


class ScheduleListResponse(BaseModel):
    destination_account_id: Annotated[str, pydantic.Field(alias="destinationAccountID")]

    mode: Mode
    r"""The operating mode for an account."""

    owner_account_id: Annotated[str, pydantic.Field(alias="ownerAccountID")]

    partner_account_id: Annotated[str, pydantic.Field(alias="partnerAccountID")]

    schedule_id: Annotated[str, pydantic.Field(alias="scheduleID")]

    source_account_id: Annotated[str, pydantic.Field(alias="sourceAccountID")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]

    description: Optional[str] = None
    r"""Simple description to place on the transfer."""

    occurrences: Optional[List[OccurrencesResponse]] = None

    recur: Optional[RecurResponse] = None

    disabled_on: Annotated[Optional[datetime], pydantic.Field(alias="disabledOn")] = (
        None
    )

    source_account: Annotated[
        Optional[PartialScheduleAccount], pydantic.Field(alias="sourceAccount")
    ] = None

    destination_account: Annotated[
        Optional[PartialScheduleAccount], pydantic.Field(alias="destinationAccount")
    ] = None

    partner_account: Annotated[
        Optional[PartialScheduleAccount], pydantic.Field(alias="partnerAccount")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "description",
                "occurrences",
                "recur",
                "disabledOn",
                "sourceAccount",
                "destinationAccount",
                "partnerAccount",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ScheduleListResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GeneratedByTypedDict(TypedDict):
    r"""The entity that generated the fee."""

    transfer_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    card_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    dispute_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    account_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    bank_account_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    invoice_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""


class GeneratedBy(BaseModel):
    r"""The entity that generated the fee."""

    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferID")] = None
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    card_id: Annotated[Optional[str], pydantic.Field(alias="cardID")] = None
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    dispute_id: Annotated[Optional[str], pydantic.Field(alias="disputeID")] = None
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    account_id: Annotated[Optional[str], pydantic.Field(alias="accountID")] = None
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    bank_account_id: Annotated[Optional[str], pydantic.Field(alias="bankAccountID")] = (
        None
    )
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    invoice_id: Annotated[Optional[str], pydantic.Field(alias="invoiceID")] = None
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "transferID",
                "cardID",
                "disputeID",
                "accountID",
                "bankAccountID",
                "invoiceID",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    GeneratedBy.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .refundstatus import RefundStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataRefundUpdatedTypedDict(TypedDict):
    account_id: str
    transfer_id: str
    refund_id: str
    status: RefundStatus


class WebhookDataRefundUpdated(BaseModel):
    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]

    refund_id: Annotated[str, pydantic.Field(alias="refundID")]

    status: RefundStatus


try:
    WebhookDataRefundUpdated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .evidencetype import EvidenceType
import io
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import FieldMetadata, MultipartFormMetadata
import pydantic
from pydantic import model_serializer
from typing import IO, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class FileTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class File(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["contentType"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateEvidenceFileMultiPartTypedDict(TypedDict):
    file: FileTypedDict
    r"""The file to upload as evidence. Valid types are [jpeg, tiff, pdf] with a limit of 4MB per file.

    The `Content-Type` header for this form part must be one of the following:
    - `image/jpeg`
    - `image/tiff`
    - `application/pdf`
    """
    evidence_type: EvidenceType


class CreateEvidenceFileMultiPart(BaseModel):
    file: Annotated[File, FieldMetadata(multipart=MultipartFormMetadata(file=True))]
    r"""The file to upload as evidence. Valid types are [jpeg, tiff, pdf] with a limit of 4MB per file.

    The `Content-Type` header for this form part must be one of the following:
    - `image/jpeg`
    - `image/tiff`
    - `application/pdf`
    """

    evidence_type: Annotated[
        EvidenceType,
        pydantic.Field(alias="evidenceType"),
        FieldMetadata(multipart=True),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .occurrence import Occurrence, OccurrenceTypedDict
from .recur import Recur, RecurTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class UpsertScheduleTypedDict(TypedDict):
    description: NotRequired[str]
    r"""Simple description of what the schedule is."""
    occurrences: NotRequired[List[OccurrenceTypedDict]]
    recur: NotRequired[RecurTypedDict]
    r"""Defines configuration for recurring transfers."""


class UpsertSchedule(BaseModel):
    description: Optional[str] = None
    r"""Simple description of what the schedule is."""

    occurrences: Optional[List[Occurrence]] = None

    recur: Optional[Recur] = None
    r"""Defines configuration for recurring transfers."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "occurrences", "recur"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class InvoicePaymentType(str, Enum):
    TRANSFER = "transfer"
    EXTERNAL = "external"
